include 'omp_lib.f90'
program LS
  use omp_lib
  implicit none
      
  logical*4               :: Vocal
  integer*4               :: ip, Np, L, Nthr, info
  real*8                  :: x, x_min, x_max, x_c, dx, Ss, Sd
  real*8,       parameter :: pi = 4.0d0*datan(1.0d0)
  character*32, parameter :: fl='(a1, a18, i10, a4, i10)'
  character*32, parameter :: f1='(f8.5, 2es12.4)'
  character*8,  parameter :: CC    = achar(13)
  integer*4,  allocatable :: nmin(:), nmax(:)
                 external :: make_qsum, make_dsum, make_bounds

  open(unit=70, file='LS.par', status='old')
  read(70,*) x_min, x_max, x_c
  read(70,*) Np
  read(70,*) L
  read(70,*) Vocal
  close(unit=70)

  Nthr = omp_get_max_threads()
  allocate(nmin(0:Nthr-1))
  allocate(nmax(0:Nthr-1))
  call make_bounds(Vocal, L, Nthr, nmin, nmax, info)
  if(info .ne. 0) then
     write(*,*) 'Make_bounds encontered a problem (fatal)'
     stop
  end if
  
  dx = (x_max - x_min) / dfloat(Np-1)
  open(unit=70, file='S.dat', status='unknown')

  write(6,'(a,i0,a,i0)') 'LattSum: Starting summation with L=', L, '; Nthr=', Nthr

  do ip=1,Np
     x = x_min + dx*dfloat(ip-1)
     if(x .ge. x_c) then
        call make_qsum(Nthr, nmin, nmax, L, x, Ss, Sd)
     else
        call make_dsum(Nthr, nmin, nmax, L, x, Ss, Sd)
     end if
     Ss = (Ss-pi)*x*x
     Sd = 1.0d0 - Sd/pi
     write(70,f1) sngl(x), sngl(Ss), sngl(Sd)
     write(6, fl, advance='NO') CC, '------ Complete : ', ip , ' of ', Np
     flush(6)
  end do
  write(*,*)
  close(unit=70)

  stop
end program LS
