  include 'blocks.f90'
  include 'formats.f90'
  include 'constants.f90'
  program PF_e
    use blocks
    use formats
    implicit none
    
    real*8    :: p, e, de, RL_a, RL_b, RL_s, RL_0, RL, dist
    real*8    :: fdist, error, D, D0, eps, Fa_i, Fb_i
    real*8    :: za_old, zb_old, sum
    real*8    :: b1, b2, beta, cfc1, cfc2, cfc3, cfc4
      
    integer*4 :: info, i, ip, ip_fail
    integer*4 :: iter, iter_inner, iter_outer

    logical*4 :: flag
    logical*4 :: use_prev = .true.    

    real*8, allocatable, dimension(:) :: y, ya, yb, z, za, zb, F
    real*8, allocatable, dimension(:) :: za_prev, zb_prev
    real*8, allocatable, dimension(:) :: Fa, Fb

    external check_pars, update_z, make_F, make_Fa, make_Fb, make_Fa_i, make_Fb_i, fdist

    Job = 'PF_e'

    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    call check_pars(info)
    write(6,f1) '******************************************'
    if(info .eq. 0) then
       write(6,f1) 'PF_e: Parameters checked for sanity'
    else if(info .gt. 0) then
       write(6,f2) 'PF_e: There were errors on input, info=', info
       stop
    else 
       write(6,f2) 'PF_e: There were warnings on input, info=', info
    end if

    p    = p_min
    beta = pi*kappa*A0
    cfc1  = ru + beta
    cfc2  = ru + two*beta
    cfc3  = four*(ru+beta)
    cfc4  = two + beta
    de    = (e_max - e_min)/dfloat(Np-1)
    if(de .le. small) then
       write(6,*) 'PF_e: de<=small; exiting; de=', de
       stop
    end if

    write(6,f1) 'PF_e computes the following functions of e at fixed kappa, p:'
    write(6,f1) 'File "L.dat"   : e, L(p,e), 1 - L(p,e)/L(p,0)'
    write(6,f1) 'File "eps.dat" : e, eps  [ = 1 / (1-4*pi*D0) -- Eq.(50a)   ]'
    write(6,f1) 'File "Dip.dat" : e, D    [ Dip.Moment/Volume -- Eq.(50b)  ]'
    write(6,f1) 'File "err.dat" : e, err  [ err = sqrt( Sum(Eq.39)^2 / N ) ]'
    write(6,f1) 'where'
    write(6,f3) '[from parameter kappa] -- kappa: ', sngl(kappa)
    write(6,f3) '[from parameter p_min] -- p    : ', sngl(p)
    write(6,f1) '******************************************'

    allocate(z(1:N))
    allocate(y(1:N))
    allocate(F(1:N))

!***************** 1)  Finding equilibrium positions at e=0'
    
!***************** 1a) Finding positions without electrostatic forces'
    F = rz; 
    F(1) = F(1) + p; F(N) = F(N) - p
    call update_z(F, z, error)
    if(error .gt. small) then
       write(6,*) 'PF_e: Error in update_z at Point A (fatal):', error
       stop
    end if

!***************** 1b) Now accounting for electrostatic interactions
    
    iter = 0
    do
       iter = iter + 1
       if(iter .gt. iter_max) then
          write(6,*) 'PF_eta: maximum number of iterations has been reached at Point A (fatal)'
          stop
       end if
       call make_F(z, F)
       F(1) = F(1) + p; F(N) = F(N) - p
       y    = z
       call update_z(F, z, error)
       if(error .gt. small) then
          write(6,*) 'PF_e: Error in update_z at point B (fatal):', error
          stop
       end if
       if(fdist(z,y) .le. th1) exit
    end do

    RL_0 = z(N) - z(1) ! Here RL is width WITH electrostatic interaction

    allocate(ya(1:N)); allocate(yb(1:N))
    allocate(za(1:N)); allocate(zb(1:N))
    allocate(Fa(1:N)); allocate(Fb(1:N))

    open(unit=71, file='L.dat'  , status='unknown')
    open(unit=72, file='eps.dat', status='unknown')
    open(unit=73, file='Dip.dat', status='unknown')
    open(unit=74, file='err.dat', status='unknown')
      
    deallocate(F)

    if(use_prev) then
       allocate(za_prev(1:N)); allocate(zb_prev(1:N))
       za_prev = z; zb_prev = z
    end if

!***************** 2)  Finding equilibrium positions at e=/=0'

    flag = .false.

!***************** 2a) Starting loop over e

    ip_loop: do ip=1,Np

       e = e_min + de*dfloat(ip-1)

       if(use_prev) then
          za = za_prev ; zb = zb_prev
       else
          za = z ; zb = z
       end if
       ya = za; yb = zb

!***************** 2b) For a fixed e, outer iteration loop

       iter_outer = 0
       iter_outer_loop: do
          iter_outer = iter_outer + 1
          if(iter_outer .gt. iter_max) then
             write(6,*) 'PF_e: maximum number of outer iterations has been reached'
             flag = .true.
             ip_fail = ip
             exit ip_loop
         end if

!***************** 2c) For a fixed e and outer iteration index, loop over atom layers

         i_loop: do i=1,N
      
!***************** 2d) For a fixed e, outer iteration index and atom layer, inner iteration loop for conditional equilibrium

            za_old = za(i); zb_old=zb(i)
            iter_inner = 0
            iter_inner_loop: do

               iter_inner = iter_inner + 1
               if(iter_inner .gt. iter_max) then
                  write(6,*) 'PF_e: maximum number of inner iterations has been reached'
                  flag = .true.
                  ip_fail = ip
                  exit ip_loop
               end if

               call make_Fa_i(i, p, e, za, zb, Fa_i)
               call make_Fb_i(i, p, e, za, zb, Fb_i)

               if     (i .eq. 1) then
                  b1 = za(2) - ru + Fa_i
                  b2 = zb(2) - ru + Fb_i
                  za(1) = (cfc1*b1 +     beta *b2) / cfc2
                  zb(1) = (beta*b1 + (ru+beta)*b2) / cfc2
               else if(i .lt. N) then
                  b1 = za(i-1) + za(i+1) + Fa_i
                  b2 = zb(i-1) + zb(i+1) + Fb_i
                  za(i) = (cfc4*b1 + beta*b2) / cfc3
                  zb(i) = (beta*b1 + cfc4*b2) / cfc3
               else
                  b1 = za(N-1) + ru + Fa_i
                  b2 = zb(N-1) + ru + Fb_i
                  za(N) = (cfc1*b1 + beta*b2) / cfc2
                  zb(N) = (beta*b1 + cfc1*b2) / cfc2
               end if

         
               dist = dsqrt((za(i)-za_old)**2 + (zb(i)-zb_old)**2)
               if(dist .le. th1) exit iter_inner_loop
               za_old = za(i); zb_old = zb(i)

            end do iter_inner_loop

         end do i_loop

         dist = (fdist(za,ya) + fdist(zb,yb))/two

         if(dist .le. th2) exit iter_outer_loop
         ya = za; yb = zb

      end do iter_outer_loop

!***************** 2e) If iterations successfull for a fixed e, find the error of equilibrium

      call make_Fa(p, e, za, zb, Fa)
      call make_Fb(p, e, za, zb, Fb)
      sum = rz
      sum = sum + (za(2) - za(1) - ru + Fa(1))**2 + (zb(2) - zb(1) - ru + Fb(1))**2
      do i=2,N-1
         sum = sum + (za(i-1) - two*za(i) + za(i+1) + Fa(i))**2 + (zb(i-1) - two*zb(i) + zb(i+1) + Fb(i))**2
      end do
      sum = sum + (za(N-1) - za(N) + ru + Fa(N))**2 + (zb(N-1) - zb(N) + ru + Fb(N))**2
      sum = dsqrt(sum/dfloat(N))
      
!***************** 2f) If iterations successfull for a fixed e, write output

      if(use_prev) then
         za_prev = za; zb_prev = zb
      end if

      RL_a = za(N) - za(1) 
      RL_b = zb(N) - zb(1)
      RL_s = RL_a + RL_b
      RL   = RL_s / two

      D = rz
      do i=1,N
         D = D + za(i)*sa(i) + zb(i)*sb(i)
      end do
      D = D / RL_s

      if(e .gt. tiny) then
         D0  =  D / e
         eps = ru / (ru - pi4 * D0)
         write(71,*) sngl(e), RL, sngl(ru - RL/RL_0)
         write(72,*) sngl(e), sngl(eps)
         write(73,*) sngl(e), sngl(D)
         write(74,*) sngl(e), sngl(sum)
      else
         write(71,*) sngl(e), RL_0, sngl(rz)
         write(73,*) sngl(e), sngl(rz)
         write(74,*) sngl(e), sngl(sum)
      end if

      if(Vocal) then
         write(6,*) 'e, RL, eps, err:', sngl(e), sngl(RL), sngl(eps), sngl(sum)
      else
         write(6, fl, advance='NO') CC, '------ Complete : ', ip , ' of ', Np
         flush(6)
      end if

   end do ip_loop

   close(unit=71); close(unit=72); close(unit=73); close(unit=74)      
   write(*,*)
      
   if(flag) then
      write(6,f4) 'PF_p: all data points up to ip=', ip_fail, ' were computed successfully'
   else
      write(6,f1) 'PF_p: all data points computed successfully'
   end if

   stop
 end program PF_e
