  include 'blocks.f90'
  include 'formats.f90'
  include 'constants.f90'
  program PF_u
    use blocks
    use formats
    implicit none
      
    real*8    :: fdist, error, u1, u2
    real*8    :: kappa_min, kappa_max, dkappa
    
    integer*4 :: info, ip, ip_fail, iter, Nh

    logical*4 :: flag

    real*8, allocatable, dimension(:) :: z, y, F
    
    external check_pars, update_z, make_F, fdist

    Job = 'PF_u'

    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    call check_pars(info)
    write(6,f1) '******************************************'
    if(info .eq. 0) then
       write(6,f1) 'PF_u: Parameters checked for sanity'
    else if(info .gt. 0) then
       write(6,f2) 'PF_u: There were errors on input, info=', info
       stop
    else 
       write(6,f2) 'PF_u: There were warnings on input, info=', info
    end if

    kappa_min = rz
    kappa_max = kappa
    dkappa    = (kappa_max-kappa_min)/dfloat(Np-1)
    Nh        = N/2

    write(6,f1) 'PF_u computes u as functions of kappa at e=p=0'
    write(6,f1) 'File "u.dat" : kappa, u1, u2'
    write(6,f1) '... where'
    write(6,f3) '[set by code         ] -- kappa_min: ', sngl(rz)
    write(6,f3) '[from parameter kappa] -- kappa_max: ', sngl(kappa)
    write(6,f1) 'u1 = (z(N)-z(1))/(N-1) ;  u2 = z(Nh+1)-z(Nh)'
    write(6,f1) '******************************************'

    allocate(z(1:N))
    allocate(y(1:N))
    allocate(F(1:N))

    open(unit=70, file='u.dat' , status='unknown')

    flag = .false.

    ip_loop: do ip=1,Np
       kappa = kappa_min + dkappa*dfloat(ip-1)

       F = rz
       call update_z(F, z, error)
       if(error .gt. small) then
          write(6,f5) 'PF_u: Error in update_z at Point A:', error
          flag = .true.
          ip_fail = ip
          exit ip_loop
       end if

       iter = 0
       do_loop: do
          iter = iter + 1
          if(iter .gt. iter_max) then
             write(6,f1) 'PF_u: Maximum number of iterations has been reached'
             flag = .true.
             ip_fail = ip
             exit ip_loop
          end if
          call make_F(z,F)
          y = z
          call update_z(F, z, error)
          if(error .gt. small) then
             write(6,f5) 'PF_u: Error in update_z at Point B:', error
             flag = .true.
             ip_fail = ip
             exit ip_loop
          end if
          if(fdist(z,y) .le. th1) exit do_loop
       end do do_loop

       u1 = ( z(N   ) - z(1 ) ) / dfloat(N-1) 
       u2 =   z(Nh+1) - z(Nh)

       write(70,*) sngl(kappa), sngl(u1), sngl(u2)

       if(Vocal) then
          write(* ,*) sngl(kappa), sngl(u1), sngl(u2)
       else
          write(6, fl, advance='NO') CC, '------ Complete : ', ip , ' of ', Np
          flush(6)
       end if

    end do ip_loop

    close(unit=70)

    write(*,*)

    if(flag) then
       write(6,f4) 'PF_u: all data points up to ip=', ip_fail, ' were computed successfully'
    else
       write(6,f1) 'PF_u: all data points computed successfully'
    end if

    stop
  end program PF_u
